﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Collections.ObjectModel;
using Uploader;
using Logger;

namespace PacketProcessor
{
    class PacketQueue
    {
        private readonly List<PolarizedPacket> packets = new List<PolarizedPacket>();

        public PacketQueue(string fullPathToLogFolder)
        {
            if (fullPathToLogFolder == null) throw new ArgumentNullException("packetQueue");
            if (!Directory.Exists(fullPathToLogFolder)) throw new ArgumentOutOfRangeException("packetQueue", "Doesn't exist");
            this.loadPacketsExceptLast(fullPathToLogFolder);
        }

        public ReadOnlyCollection<PolarizedPacket> Packets
        {
            get { return new ReadOnlyCollection<PolarizedPacket>(this.packets); }
        }

        private void loadPacketsExceptLast(string fullPathToLogFolder)
        {
            var directory = new DirectoryInfo(fullPathToLogFolder);
            var allXmlFiles = directory.GetFiles()
                .Where(x => x.Extension.ToLower() == ".xml")
                .OrderBy(x => x.Name)
                .ToList();
            var files = allXmlFiles
                .Take(allXmlFiles.Count - 1) // ignore last one as it might not be written yet
                .ToList();
            foreach (var file in files)
            {
                var filename = file.FullName;
                var packet = filename.DeserializeObject<PolarizedPacket>();
                this.packets.Add(packet);
            }
        }
    }
}
