﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace PacketProcessor
{
    public class SliceFileFinder
    {
        public static string FindLatestSliceFile(string fullPathToDataFolder)
        {
            var directory = new DirectoryInfo(fullPathToDataFolder);
            var latestFiles = directory.GetFiles()
                .Where(x => x.Extension.ToLower() == ".dat")
                .OrderByDescending(x => x.Name)
                .ToList();
            if (latestFiles.Count > 0)
            {
                var latestFile = latestFiles[0];
                if (latestFile.Length < Program.FULL_TIMESLICE_FILE_LENGTH)
                {
                    return latestFile.FullName;
                }
                else
                {
                    var unparsedDate = Path.GetFileNameWithoutExtension(latestFile.FullName);
                    var parsedDate = DateTime.Parse(unparsedDate);
                    var newDate = parsedDate.AddDays(1);
                    var newFileName = FilenameByDate(fullPathToDataFolder, newDate);
                    MakeZeroByteFile(newFileName);
                    return newFileName;
                }
            }
            var filename = FilenameByDate(fullPathToDataFolder, DateTime.Today);
            MakeZeroByteFile(filename);
            return filename;
        }

        public static string FilenameByDate(string fullPathToDataFolder, DateTime date)
        {
            var filename = Path.Combine(fullPathToDataFolder, date.ToString("yyyy-MM-dd") + ".dat");
            return filename;
        }

        public static void MakeZeroByteFile(string filename)
        {
            using (var fileStream = File.Create(filename)) 
            {
                fileStream.Close();
            };
        }
    }
}
