﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Uploader
{
    public class PacketGetter
    {
        private readonly ISerialPortController serialPortController;
        private readonly byte commandByte;

        public PacketGetter(
            ISerialPortController serialPortController,
            byte commandByte)
        {
            if (serialPortController == null) throw new ArgumentNullException("serialPortController");
            if (commandByte < 0xFC) throw new ArgumentOutOfRangeException("commandByte");
            this.serialPortController = serialPortController;
            this.commandByte = commandByte;
        }

        public bool TryReadPolarizedPacket(out PolarizedPacket polarizedPacket, out Exception exception)
        {
            try
            {
                var wakeupResponse = this.serialPortController.SendReceive(new byte[] { this.commandByte }, 1);
                if (wakeupResponse.Length < 1)
                {
                    throw new Exception("Didn't respond to wakeup byte.");
                }
                if (wakeupResponse.Length > 1)
                {
                    throw new Exception("Response to wakeup byte was too long.");
                }
                if (wakeupResponse[0] != 0xFC)
                {
                    throw new Exception("Wakeup byte response should be 0xFC but was: 0x" + wakeupResponse[0].ToString("X"));
                }
                var spkLength = PolarizedPacket.PACKET_LENGTH + 1;
                var spkResponse = this.serialPortController.SendReceive(getBytes("SPK"), spkLength);
                this.serialPortController.ClosePortIfOpen();
                if (spkResponse.Length < spkLength)
                {
                    throw new Exception("Response to SPK command was too short.");
                }
                if (spkResponse.Length > spkLength)
                {
                    throw new Exception("Response to SPK command was too long.");
                }
                if (spkResponse[0] != 0xFC)
                {
                    throw new Exception("SPK command response should be 0xFC but was: 0x" + spkResponse[0].ToString("X"));
                }
                var packetData = spkResponse.Skip(1).ToArray();
                polarizedPacket = new PolarizedPacket(DateTime.Now, packetData);
                exception = null;
                return true;
            }
            catch (Exception ex)
            {
                polarizedPacket = null;
                exception = ex;
                return false;
            }
        }

        private static byte[] getBytes(string str)
        {
            return System.Text.Encoding.UTF8.GetBytes(str.ToCharArray());
        }
    }
}
