﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Uploader
{
    class Program
    {
        static void Main(string[] args)
        {
            var parameters = new Parameters(args);
            if (parameters.PrintUsage)
            {
                printUsage();
                return;
            }
            using (var serialPortController = new SerialPortController(parameters.ComPort))
            {
                var fcPacketGetter = new PacketGetter(serialPortController, parameters.CommandByte);
                PolarizedPacket polarizedPacket;
                Exception exception;
                if (fcPacketGetter.TryReadPolarizedPacket(out polarizedPacket, out exception))
                {
                    Console.WriteLine("Success (0x{0:X}):", parameters.CommandByte);
                    Console.WriteLine(polarizedPacket.ToString());
                }
                else
                {
                    Console.WriteLine("Error getting data:");
                    Console.WriteLine(exception.Message);
                }
            }
        }

        static void printUsage()
        {
            Console.WriteLine("Usage: Uploader -port COM1 -commandbyte FC");
        }

        class Parameters
        {
            public Parameters(string[] args)
            {
                this.ComPort = string.Empty;
                if (args == null)
                {
                    PrintUsage = true;
                    return;
                }
                if (args.Length != 4)
                {
                    PrintUsage = true;
                    return;
                }
                if (args.Any(x => x.ToLower() == "-h" || x == "/?"))
                {
                    PrintUsage = true;
                    return;
                }
                if (args[0].ToLower() != "-port")
                {
                    PrintUsage = true;
                    return;
                }
                if (args[2].ToLower() != "-commandbyte")
                {
                    PrintUsage = true;
                    return;
                }
                if (!args[1].ToLower().StartsWith("com"))
                {
                    PrintUsage = true;
                    return;
                }
                if (!(args[3].ToLower() == "fc"
                    || args[3].ToLower() == "fd"
                    || args[3].ToLower() == "fe"
                    || args[3].ToLower() == "ff"))
                {
                    PrintUsage = true;
                    return;
                }
                this.ComPort = args[1].ToUpper();
                this.CommandByte = parseCommandByte(args[3]);
            }

            public bool PrintUsage { get; private set; }
            public byte CommandByte { get; private set; }
            public string ComPort { get; private set; }

            private static byte parseCommandByte(string cb)
            {
                switch (cb.ToLower())
                {
                    case "fc":
                        return 0xFC;
                    case "fd":
                        return 0xFD;
                    case "fe":
                        return 0xFE;
                    case "ff":
                        return 0xFF;
                    default:
                        return 0xFC;
                }
            }
        }
    }
}
