﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using PacketProcessor;
using System.IO;

namespace Verifier
{
    class Program
    {
        static string fullPathToDataFolder;

        static void Main(string[] args)
        {
            fullPathToDataFolder = PacketProcessor.Program.FullPathToDataFolder();

            var parameters = new Parameters(args);
            if (parameters.PrintUsage)
            {
                printUsage();
                return;
            }
            var filename = SliceFileFinder.FilenameByDate(fullPathToDataFolder, parameters.Date);
            var fileInfo = new FileInfo(filename);
            var slices = fileInfo.Length / TimeSlice.BYTES_PER_SLICE;

            using (var fileStream = new FileStream(filename, FileMode.Open, FileAccess.Read))
            using (var reader = new BinaryReader(fileStream))
            {
                for (var i = 0; i < slices; i++)
                {
                    var bytes = reader.ReadBytes(TimeSlice.BYTES_PER_SLICE);
                    var timeSlice = new TimeSlice(bytes);
                    var sliceEnding = parameters.Date.AddSeconds(i * 10).ToString("HH:mm:ss");
                    Console.Write("Slice: " + i + " ending " + sliceEnding + ", ");
                    if (parameters.Voltage)
                    {
                        Console.WriteLine("Voltage: " + timeSlice.Voltage);
                    }
                    else
                    {
                        Console.WriteLine("Channel " + parameters.Channel + ": " + timeSlice.WattSeconds(parameters.Channel));
                    }
                }
                reader.Close();
                fileStream.Close();
            }
            Console.ReadKey();
        }

        private static void printUsage()
        {
            Console.WriteLine("Usage: Verifier.exe -date yyyy-MM-dd -voltage");
            Console.WriteLine("Usage: Verifier.exe -date yyyy-MM-dd -channel #");
            Console.WriteLine("... where # is 0 to 13");
        }

        class Parameters
        {
            public Parameters(string[] args)
            {
                this.Date = DateTime.Today.AddDays(-1);
                if (args == null)
                {
                    PrintUsage = true;
                    return;
                }
                if (args.Length < 3)
                {
                    PrintUsage = true;
                    return;
                }
                if (args.Any(x => x.ToLower() == "-h" || x == "/?"))
                {
                    PrintUsage = true;
                    return;
                }
                if (args[0].ToLower() != "-date")
                {
                    PrintUsage = true;
                    return;
                }
                DateTime date;
                if (!DateTime.TryParse(args[1], out date))
                {
                    PrintUsage = true;
                    return;
                }
                this.Date = date.Date;

                if (args[2].ToLower() == "-voltage")
                {
                    this.Voltage = true;
                }
                else if (args[2].ToLower() == "-channel")
                {
                    if (args.Length < 3)
                    {
                        PrintUsage = true;
                        return;
                    }
                    int channel;
                    if (!int.TryParse(args[3], out channel))
                    {
                        PrintUsage = true;
                        return;
                    }
                    if (channel < 0 || channel > 13)
                    {
                        PrintUsage = true;
                        return;
                    }
                    this.Channel = channel;
                }
                else
                {
                    PrintUsage = true;
                    return;
                }
            }

            public DateTime Date { get; private set; }
            public bool PrintUsage { get; private set; }

            public bool Voltage { get; private set; }
            public int Channel { get; private set; }
        }
    }
}
