﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Viewer
{
    class Channel
    {
        private readonly Action onSelectionChanged;
        private bool selected = false;
        private int index = 0;

        public Channel(
            Action onSelectionChanged)
        {
            if (onSelectionChanged == null) throw new ArgumentNullException("onSelectionChanged");
            this.onSelectionChanged = onSelectionChanged;
        }

        public bool Selected {
            get { return this.selected; }
            set
            {
                if (value != this.selected)
                {
                    this.selected = value;
                    this.onSelectionChanged();
                }
            }
        }

        public int Index 
        {
            get { return this.index; }
            set
            {
                this.index = value;
                if (value == 0)
                {
                    this.selected = true; // default to main panel being selected
                }
            }
        }
        public string Name { get; set; }

        public void SetSelected(bool value)
        {
            this.selected = value;
        }

        public double TotalEnergy_kWh 
        {
            get { return this.OnPeakEnergy_kWh + this.MidPeakEnergy_kWh + this.OffPeakEnergy_kWh; }
        }
        public double OnPeakEnergy_kWh { get; set; }
        public double MidPeakEnergy_kWh { get; set; }
        public double OffPeakEnergy_kWh { get; set; }
    }
}
