﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Viewer
{
    public enum TimeOfUse
    {
        OffPeak,
        MidPeak,
        OnPeak,
    }

    public static class TimeOfUseHelper
    {
        public static TimeOfUse ToTimeOfUse(this DateTime dateTime)
        {
            // 7 pm to 7 am is always off peak
            if (dateTime.Hour < 7) return TimeOfUse.OffPeak;
            if (dateTime.Hour >= 19) return TimeOfUse.OffPeak;

            if (isWeekendOrHoliday(dateTime)) return TimeOfUse.OffPeak;

            // summer pricing May 1 to Oct 31
            if (dateTime.Month >= 5 && dateTime.Month <= 10)
            {
                // 11 to 5 is on-peak
                if (dateTime.Hour >= 11 && dateTime.Hour < 17) return TimeOfUse.OnPeak;
                return TimeOfUse.MidPeak;
            }

            // winter pricing, reversed from summer mid/on-peak
            // 11 to 5 is mid-peak
            if (dateTime.Hour >= 11 && dateTime.Hour < 17) return TimeOfUse.MidPeak;
            return TimeOfUse.OnPeak;
        }

        private static bool isWeekendOrHoliday(DateTime dateTime)
        {
            var dayOfWeek = dateTime.DayOfWeek;
            if (dayOfWeek == DayOfWeek.Saturday || dayOfWeek == DayOfWeek.Sunday) return true;

            // New Year's Day
            if (dateTime.Month == 1 && dateTime.Day == 1) return true;

            // Family Day - 3rd Monday in Feb
            if (dateTime.Month == 2 && dayOfWeek == DayOfWeek.Monday && dayOccurrence(dateTime) == 3) return true;

            // Good Friday
            if (dateTime == goodFriday(dateTime.Year)) return true;

            // Victoria Day - Monday before May 25th
            if (dateTime.Month == 5 && dayOfWeek == DayOfWeek.Monday && dateTime.Day <= 24 && dateTime.Day >= 18) return true;

            // Canada Day
            if (dateTime.Month == 7 && dateTime.Day == 1) return true;

            // Civic Holiday - first Monday in August
            if (dateTime.Month == 8 && dayOfWeek == DayOfWeek.Monday && dayOccurrence(dateTime) == 1) return true;

            // Labour Day - first Monday in September
            if (dateTime.Month == 9 && dayOfWeek == DayOfWeek.Monday && dayOccurrence(dateTime) == 1) return true;

            // Thanksgiving - second Monday in October
            if (dateTime.Month == 10 && dayOfWeek == DayOfWeek.Monday && dayOccurrence(dateTime) == 2) return true;

            // Christmas
            if (dateTime.Month == 12 && dateTime.Day == 25) return true;

            // Boxing Day
            if (dateTime.Month == 12 && dateTime.Day == 26) return true;

            return false;
        }

        private static int dayOccurrence(DateTime date)
        {
            return (int)Math.Ceiling(date.Day / 7.0);
        }

        private static DateTime goodFriday(int year)
        {
            return easterSunday(year).AddDays(-2);
        }

        private static DateTime easterSunday(int year)
        {
            int day = 0;
            int month = 0;

            int g = year % 19;
            int c = year / 100;
            int h = (c - (int)(c / 4) - (int)((8 * c + 13) / 25) + 19 * g + 15) % 30;
            int i = h - (int)(h / 28) * (1 - (int)(h / 28) * (int)(29 / (h + 1)) * (int)((21 - g) / 11));

            day = i - ((year + (int)(year / 4) + i + 2 - c + (int)(c / 4)) % 7) + 28;
            month = 3;

            if (day > 31)
            {
                month++;
                day -= 31;
            }

            return new DateTime(year, month, day);
        }
    }
}
